#!/usr/bin/python
"""
for ElchemeaAnalytics
cgra 2012-09-13

1. get f, re, im from stdin (tab-delimited)
2. calculate and print kk-estimated inductance to stdout

Usage:
    script_that_prints_impedance_data | get_L_by_kk.py 10
where 10 is the number of points to use for polyfit (recommended 1 decade); If not 
given, defaults to 10 or the total number of points in the data, whichever is lower.
"""

import sys
import select
import numpy as np
#from scipy.integrate import trapz

def trapz(y,x):
    dx = x[1:]-x[:-1]
    dy = ( y[1:]+y[:-1] )/2
    return np.sum(np.dot(dx,dy))

def ln(x):
    return np.log(x)
    
def getL_kk(data, M=10):
    """
    Function to estimate inductance correction parameter
    by kk-transforming the final M real data points
    to imaginary. Compare to measured imaginary
    and fit difference with straight line in semilog
    
    Adapted from Matlab functions originally written by 
    Jakob Heide Joergensen, Risoe, 2007-08-24

    and ported to python by 
    Chris Graves (cgra), Risoe DTU, 2011-12-02
    """
    d = data
    if data.f[0] > data.f[1]:
        d = np.sort(data)

    re = d.re
    im = d.im
    omega = 2.0*np.pi*d.f
    n = len(omega)
    imkk  = kkre2im(re,omega)
    j     = np.arange(n-M,n)

    imdif = im[j]-imkk[j]
    coeff = np.polyfit(omega[j],imdif,1)
    L     = coeff[0]
    
    return L


def kkre2im(re,omega):
    """
    Perform Kramers-Kronig transform of real data re and
    angular frequencies omega to get imaginary parts. Handle
    division by zero by letting integrand be the mean
    of the adjoining values.
    
    Matlab function by Jakob Heide Joergensen, Risoe, 2007-08-24. 

    Ported to Python by Chris Graves (cgra), Risoe DTU, 2011-12-02.
    """
    np.seterr(all='ignore') # ignore division by zero
    
    imkk = np.zeros(len(re)) # set up vector for reconstructed imag parts
    N = len(re)           # number of data points
    
    # for each data point, do the transform
    for k in xrange(0,N):
        # compute the integrand
        inte = (re - re[k])/(omega**2 - omega[k]**2)
        
        # handle division by zero by taking mean and end-points
        # by setting equal to adjoining point
        if k == 0: 
            inte[k] = inte[1] 
        elif k == N-1:
            inte[k] = inte[N-2]
        else:
            inte[k] = ( inte[k-1] + inte[k+1] )/2
    
        # integrate
        intecur = trapz(inte*omega*ln(10),np.log10(omega)) # y,x which is reversed from Jakob's integrate() function

        # scale by constant to get final imaginary partz
        
        imkk[k] = 2.0*omega[k]/np.pi*intecur

    return imkk


# --- "MAIN" ---

if select.select([sys.stdin,],[],[],0.0)[0]:
    data = np.recfromtxt(sys.stdin, names=['f', 're', 'im'], delimiter='\t')
    
    if len(sys.argv) < 2:
        #print 'Number of points for polyfit not passed in as an argument. Assuming 10 points.'
        nr_points = 10
    else:
        nr_points = int(sys.argv[1])
    
    if nr_points > len(data):
        nr_points = len(data)
    
    print getL_kk(data, nr_points)
else:
    print -1

